#include "myevent.h"
#include <cairomm/context.h>
#include <gtkmm/label.h>
#include <iostream>


// default Model Framing and window parameters
static Frame default_frame = {-150., 150., -100., 100., 1.5, 300, 200}; 

constexpr int area_side(200);

static void draw_frame(const Cairo::RefPtr<Cairo::Context>& cr, Frame frame);
static void orthographic_projection(const Cairo::RefPtr<Cairo::Context>& cr, 
									Frame frame);

MyArea::MyArea(): empty(false)
{
	set_content_width(area_side);
	set_content_height(area_side);
	
	set_draw_func(sigc::mem_fun(*this, &MyArea::on_draw));
}

MyArea::~MyArea()
{
}

void MyArea::clear()
{
	empty = true; 
	queue_draw();
}

void MyArea::draw()
{
	empty = false;
	queue_draw();
}

// defining the Model space frame to visualize in the window canvas
void MyArea::setFrame(Frame f)
{
	if((f.xMin <= f.xMax) and (f.yMin <= f.yMax) and (f.height > 0))
	{
		f.asp = f.width/f.height;
		frame = f;
	}
	else
		std::cout << "incorrect Model framing or window parameters" << std::endl;
} 

void MyArea::adjustFrame(int width, int height)
{
	frame.width  = width;
	frame.height = height;

	// Preventing distorsion by adjusting the frame (cadrage)
	// to have the same proportion as the graphical area
	
    // use the reference framing as a guide for preventing distortion
    double new_aspect_ratio((double)width/height);
    if( new_aspect_ratio > default_frame.asp)
    { // keep yMax and yMin. Adjust xMax and xMin
	    frame.yMax = default_frame.yMax ;
	    frame.yMin = default_frame.yMin ;	
	  
	    double delta(default_frame.xMax - default_frame.xMin);
	    double mid((default_frame.xMax + default_frame.xMin)/2);
        // the new frame is centered on the mid-point along X
	    frame.xMax = mid + 0.5*(new_aspect_ratio/default_frame.asp)*delta ;
	    frame.xMin = mid - 0.5*(new_aspect_ratio/default_frame.asp)*delta ;		  	  
    }
    else
    { // keep xMax and xMin. Adjust yMax and yMin
	    frame.xMax = default_frame.xMax ;
	    frame.xMin = default_frame.xMin ;
	  	  
 	    double delta(default_frame.yMax - default_frame.yMin);
	    double mid((default_frame.yMax + default_frame.yMin)/2);
        // the new frame is centered on the mid-point along Y
	    frame.yMax = mid + 0.5*(default_frame.asp/new_aspect_ratio)*delta ;
	    frame.yMin = mid - 0.5*(default_frame.asp/new_aspect_ratio)*delta ;		  	  
    }
}




void MyArea::on_draw(const Cairo::RefPtr<Cairo::Context>& cr, int width, int height)
{
	if(not empty)   // drawing in the Model space
	{
		// adjust the frame (cadrage) to prevent distortion 
		adjustFrame(width, height);
		draw_frame(cr, frame);              // drawing the drawingArea space
	
		orthographic_projection(cr, frame); // set the transformation MODELE to GTKmm
	
		//set width and color
		cr->set_line_width(10);
		cr->set_source_rgb(0.8, 0., 0.0);
	
		//Now we can draw directly in the Model space
		
		// first line
		cr->move_to(-50., -50);
		cr->line_to(50. , 50);
		//draw second line
		cr->move_to(-50. , 50);
		cr->line_to(50.  ,-50.);
	
		cr->stroke();
	}
	else
	{
		std::cout << "Empty !" << std::endl;
	}
}

static void draw_frame(const Cairo::RefPtr<Cairo::Context>& cr, Frame frame)
{
	//display a rectangular frame around the drawing area
	cr->set_line_width(10.0);
	// draw greenish lines
	cr->set_source_rgb(0., 0.7, 0.2);
	cr->rectangle(0,0, frame.width, frame.height);
	cr->stroke();
}

static void orthographic_projection(const Cairo::RefPtr<Cairo::Context>& cr, 
								    Frame frame)
{
	// déplace l'origine au centre de la fenêtre
	cr->translate(frame.width/2, frame.height/2);
  
	// normalise la largeur et hauteur aux valeurs fournies par le cadrage
	// ET inverse la direction de l'axe Y
	cr->scale(frame.width/(frame.xMax - frame.xMin), 
             -frame.height/(frame.yMax - frame.yMin));
  
	// décalage au centre du cadrage
	cr->translate(-(frame.xMin + frame.xMax)/2, -(frame.yMin + frame.yMax)/2);
}

//------------ end of MyArea class definition -----------------

MyEvent::MyEvent():
	m_Main_Box(Gtk::Orientation::HORIZONTAL, 0),
	m_Buttons_Box(Gtk::Orientation::VERTICAL, 2),
	m_Button_Clear("Clear"),
	m_Button_Draw("Draw")
{
	set_title("GUI without distorsion");

	set_child(m_Main_Box);

	m_Main_Box.append(m_Buttons_Box);
	m_Main_Box.append(m_Area);
	
	// allow the drawingArea expand to the window size
	m_Area.set_expand();
	
	m_Buttons_Box.append(m_Button_Clear);
	m_Buttons_Box.append(m_Button_Draw);
	
	m_Button_Clear.signal_clicked().connect(
		sigc::mem_fun(*this, &MyEvent::on_button_clicked_clear));
		
    m_Button_Draw.signal_clicked().connect(
		sigc::mem_fun(*this, &MyEvent::on_button_clicked_draw));
}

void MyEvent::on_button_clicked_clear()
{
	std::cout << "Clear" << std::endl;
	m_Area.clear();
}

void MyEvent::on_button_clicked_draw()
{
	std::cout << "Draw" << std::endl;
	m_Area.draw();
}
